﻿using Hotline.Application.Contracts.Validators.FlowEngine;
using Hotline.Caching.Interfaces;
using Hotline.Caching.Services;
using Hotline.FlowEngine;
using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.Identity.Accounts;
using Hotline.Identity.Roles;
using Hotline.Orders;
using Hotline.SeedData;
using Hotline.Settings;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.FlowEngine.Definition;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Identity;
using Hotline.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Http;
using SqlSugar;
using System.Threading;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Extensions;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Application.FlowEngine;

public class WorkflowApplication : IWorkflowApplication, IScopeDependency
{
    private readonly IDefinitionDomainService _definitionDomainService;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly IOrderDomainService _orderDomainService;
    private readonly IWorkflowRepository _workflowRepository;
    private IRepository<WorkflowDefinition> _definitionRepository;
    private readonly IRepository<WorkflowAssign> _workflowAssignRepository;
    private readonly IRepository<User> _userRepository;
    private readonly IAccountRepository _accountRepository;
    private readonly ISystemOrganizeRepository _organizeRepository;
    private readonly IRepository<Role> _roleRepository;
    private readonly IWfModuleCacheManager _wfModuleCacheManager;
    private readonly ISystemDomainService _systemDomainService;
    private readonly IUserDomainService _userDomainService;
    private readonly IAccountDomainService _accountDomainService;
    private readonly ISessionContext _sessionContext;
    private readonly IMapper _mapper;

    public WorkflowApplication(
        IDefinitionDomainService definitionDomainService,
        IWorkflowDomainService workflowDomainService,
        IOrderDomainService orderDomainService,
        IWorkflowRepository workflowRepository,
        IRepository<WorkflowDefinition> definitionRepository,
        IRepository<WorkflowAssign> workflowAssignRepository,
        IRepository<User> userRepository,
        IAccountRepository accountRepository,
        ISystemOrganizeRepository organizeRepository,
        IRepository<Role> roleRepository,
        IWfModuleCacheManager wfModuleCacheManager,
        ISystemDomainService systemDomainService,
        ISessionContext sessionContext,
        IMapper mapper
        )
    {
        _definitionDomainService = definitionDomainService;
        _workflowDomainService = workflowDomainService;
        _orderDomainService = orderDomainService;
        _workflowRepository = workflowRepository;
        _definitionRepository = definitionRepository;
        _workflowAssignRepository = workflowAssignRepository;
        _userRepository = userRepository;
        _accountRepository = accountRepository;
        _organizeRepository = organizeRepository;
        _roleRepository = roleRepository;
        _wfModuleCacheManager = wfModuleCacheManager;
        _systemDomainService = systemDomainService;
        _sessionContext = sessionContext;
        _mapper = mapper;
    }

    public async Task<string> StartWorkflowAsync(StartWorkflowDto dto, string externalId, CancellationToken cancellationToken)
    {
        var validator = new StartWorkflowDtoValidator();
        var validResult = validator.Validate(dto);
        if (!validResult.IsValid)
            throw new UserFriendlyException(
                $"非法参数, {string.Join(',', validResult.Errors.Select(d => d.ErrorMessage))}");

        var wfModule = await GetWorkflowModuleAsync(dto.DefinitionModuleCode, cancellationToken);
        var definition = wfModule.Definition;
        if (definition == null)
            throw new UserFriendlyException("无效模板编码");
        if (definition.Status is not EDefinitionStatus.Enable)
            throw new UserFriendlyException("该模板不可用");

        //如果发起会签需检查是否支持发起会签
        var startStepDefine = definition.FindStartStepDefine();

        var firstStepDefine = startStepDefine.InstanceMode is EInstanceMode.Dynamic
            ? startStepDefine
            : definition.FindStepDefine(dto.NextStepCode);
        if (firstStepDefine is null)
            throw new UserFriendlyException("未查询到下一步节点配置");

        //1. 如果不是按角色指派，handlers必填 2. 如果按角色指派，handlers可以不选
        if (firstStepDefine.HandlerType is not EHandlerType.Role && !dto.NextHandlers.Any())
            throw UserFriendlyException.SameMessage("未指派办理人");

        if (dto.IsStartCountersign)
        {
            if (!startStepDefine.CanStartCountersign)
                throw new UserFriendlyException("当前节点不支持发起会签");
            //if (startStepDefine.HandlerType is EHandlerType.Role)
            //    throw new UserFriendlyException("当前节点不支持发起会签");
            //即使当前节点支持发起会签，但下一节点为信息汇总节点、结束节点时也不可发起会签
            if (firstStepDefine.StepType is EStepType.Summary or EStepType.End)
                throw new UserFriendlyException("下一节点不允许发起会签");
            //下一节点是会签汇总节点也不允许发起会签
            if (dto.BackToCountersignEnd)
                throw new UserFriendlyException("下一节点不允许发起会签");
        }

        var workflow = await _workflowDomainService.CreateWorkflowAsync(wfModule, dto.Title,
            _sessionContext.RequiredUserId, _sessionContext.RequiredOrgId, externalId, cancellationToken);

        var flowAssignInfo = await GetFlowAssignInfoAsync(firstStepDefine, dto.IsStartCountersign, dto.NextHandlers,
            cancellationToken);

        await _workflowDomainService.StartAsync(workflow, dto, startStepDefine, firstStepDefine, flowAssignInfo, cancellationToken);

        ////更新接办部门(详情页面展示)
        //await AddOrUpdateAssignAsync(workflow, dto, nextStepBoxDefine, cancellationToken);

        return workflow.Id;
    }

    /// <summary>
    /// 流转至下一节点（节点办理）
    /// </summary>
    public async Task NextAsync(NextWorkflowDto dto, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true, withCountersigns: true,
            cancellationToken: cancellationToken);
        //await NextAsync(workflow, dto, dto.ExpiredTime, dto.IsStartCountersign, cancellationToken);

        ////未超期工单,节点超期时间不能小于当前时间，不能大于流程整体超期时间
        //if (workflow.ExpiredTime > DateTime.Now &&
        //    (dto.StepExpiredTime <= DateTime.Now || dto.StepExpiredTime > workflow.ExpiredTime))
        //    throw UserFriendlyException.SameMessage("节点期满时间无效");

        var currentStep = _workflowDomainService.FindCurrentStepWaitForHandle(workflow);
        if (currentStep.Status is EWorkflowStepStatus.Handled)
            throw new UserFriendlyException("该状态不支持继续办理");

        var currentStepDefine = _workflowDomainService.GetStepDefine(workflow.WorkflowDefinition, currentStep.Code);

        StepDefine nextStepDefine;
        if ((currentStep.InstanceMode is EInstanceMode.Dynamic && !DynamicShouldTerminal(currentStep))
            || currentStep.IsInCountersign()
            || dto.IsStartCountersign)
        {
            //下一步配置为当前节点配置
            nextStepDefine = currentStepDefine;
        }
        else
        {
            //下一步配置为下一步节点配置
            nextStepDefine = _workflowDomainService.GetStepDefine(workflow.WorkflowDefinition, dto.NextStepCode);
        }

        //需求：按角色选择办理人可以不选，表示该角色下所有人都可以办理，同时依据配置：是否本部门人办理显示待选办理人。角色下只要一人办理即可（即:角色下不发起会签）
        if (nextStepDefine.HandlerType != EHandlerType.Role && !dto.NextHandlers.Any())
            throw new UserFriendlyException("未指定节点处理者");

        if (dto.IsStartCountersign)
        {
            if (!currentStepDefine.CanStartCountersign)
                throw new UserFriendlyException("当前节点不支持发起会签");
            //if (currentStepDefine.HandlerType is EHandlerType.Role)
            //    throw new UserFriendlyException("当前节点不支持发起会签");
            //即使当前节点支持发起会签，但下一节点为信息汇总节点、结束节点时也不可发起会签
            if (nextStepDefine.StepType is EStepType.Summary or EStepType.End)
                throw new UserFriendlyException("下一节点不允许发起会签");
            //下一节点是会签汇总节点也不允许发起会签
            if (dto.BackToCountersignEnd)
                throw new UserFriendlyException("下一节点不允许发起会签");
        }

        var flowAssignMode = await GetFlowAssignInfoAsync(nextStepDefine, dto.IsStartCountersign, dto.NextHandlers,
            cancellationToken);

        await _workflowDomainService.NextAsync(workflow, currentStep, dto, nextStepDefine, flowAssignMode, dto.StepExpiredTime,
            cancellationToken);
    }

    /// <summary>
    /// 撤回至任意节点
    /// </summary>
    public async Task RecallAsync(RecallDto dto, CancellationToken cancellationToken)
    {
        var workflow =
            await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true,
                cancellationToken: cancellationToken);

        await _orderDomainService.ReadyToRecallAsync(workflow.ExternalId, cancellationToken);

        var targetStepDefine = _workflowDomainService.GetStepDefine(workflow.WorkflowDefinition, dto.NextStepCode);
        //var isStartCountersign = targetStepDefine.CouldPrevStartCountersign(dto.NextHandlers.Count);
        var flowAssignMode = await GetFlowAssignInfoAsync(targetStepDefine, dto.IsStartCountersign, dto.NextHandlers,
            cancellationToken);
        await _workflowDomainService.RecallAsync(workflow, dto, targetStepDefine, flowAssignMode, cancellationToken);
    }

    /// <summary>
    /// 跳转至任意节点
    /// </summary>
    public async Task JumpAsync(RecallDto dto, CancellationToken cancellationToken)
    {
        var workflow =
            await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true,
                cancellationToken: cancellationToken);

        await _orderDomainService.ReadyToRecallAsync(workflow.ExternalId, cancellationToken);

        var targetStepDefine = _workflowDomainService.GetStepDefine(workflow.WorkflowDefinition, dto.NextStepCode);
        //var isStartCountersign = targetStepDefine.CouldPrevStartCountersign(dto.NextHandlers.Count);
        var flowAssignMode = await GetFlowAssignInfoAsync(targetStepDefine, dto.IsStartCountersign, dto.NextHandlers,
            cancellationToken);
        await _workflowDomainService.JumpAsync(workflow, dto, targetStepDefine, flowAssignMode, cancellationToken);
    }

    /// <summary>
    /// 重办
    /// </summary>
    public async Task RedoAsync(RecallDto dto, CancellationToken cancellationToken)
    {
        var workflow =
            await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true,
                cancellationToken: cancellationToken);
        var targetStepDefine = _workflowDomainService.GetStepDefine(workflow.WorkflowDefinition, dto.NextStepCode);
        var flowAssignInfo = await GetFlowAssignInfoAsync(targetStepDefine, dto.IsStartCountersign, dto.NextHandlers,
            cancellationToken);
        await _workflowDomainService.RedoAsync(workflow, dto, targetStepDefine, flowAssignInfo, cancellationToken);
    }

    public async Task<DefinedStepDto> GetStartOptionsAsync(string moduleCode, CancellationToken cancellationToken)
    {
        var wfModule = await GetWorkflowModuleAsync(moduleCode, cancellationToken);
        var definition = wfModule.Definition;
        if (definition == null)
            throw new UserFriendlyException("无效模板编码");
        if (definition.Status is not EDefinitionStatus.Enable)
            throw new UserFriendlyException("该模板不可用");

        var startStep = definition.Steps.FirstOrDefault(d => d.StepType == EStepType.Start);
        if (startStep == null)
            throw new UserFriendlyException("未正确配置发起人节点");
        var firstStepDefines = definition.FindStepDefines(startStep.NextSteps.Select(d => d.Code));
        if (!firstStepDefines.Any())
            throw new UserFriendlyException("未正确配置首个办理节点");

        return new DefinedStepDto
        {
            DefinitionId = definition.Id,
            Steps = firstStepDefines
            // Components = firstStepDefine.Components
        };
    }

    /// <summary>
    /// 根据节点配置查询待选参数
    /// </summary>
    /// <param name="stepDefine"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="ArgumentOutOfRangeException"></exception>
    public async Task<IReadOnlyList<Kv>> GetNextStepOptionsAsync(StepDefineBasic stepDefine,
        CancellationToken cancellationToken)
    {
        //if (stepDefine.StepType == EStepType.End)
        //    throw new UserFriendlyException("结束节点无待选项");

        //var handlers = new List<Kv>();
        //switch (stepDefine.HandlerType)
        //{
        //    // case EHandlerType.AssignedUser:
        //    //     var users = await _userRepository.Queryable()
        //    //         .Where(d => stepDefine.HandlerTypeItems.Select(x => x.Id).Contains(d.Id))
        //    //         .ToListAsync(cancellationToken);
        //    //     handlers = users.Select(d => new Kv(d.Id, d.Name)).ToList();
        //    //     break;
        //    // case EHandlerType.AssignedOrg:
        //    //     var orgs = await _organizeRepository.QueryAsync(d =>
        //    //         d.IsEnable &&
        //    //         stepDefine.HandlerTypeItems.Select(d => d.Id).Contains(d.Id));
        //    //     handlers = orgs.Select(d => new Kv(d.Id, d.OrgName))
        //    //         .ToList();
        //    //     break;
        //    case EHandlerType.AssignedUser:
        //    case EHandlerType.AssignedOrg:
        //        handlers = stepDefine.HandlerTypeItems;
        //        break;
        //    case EHandlerType.Role:
        //        //当前操作人所属部门的下级部门并且属于配置包含角色
        //        var roles = await _roleRepository.Queryable()
        //            .Includes(d => d.Accounts.Where(x => !x.IsDeleted && x.Status == EAccountStatus.Normal).ToList(),
        //                x => x.User)
        //            .Where(d => stepDefine.HandlerTypeItems.Select(x => x.Key).Contains(d.Name))
        //            .ToListAsync(cancellationToken);
        //        var users1 = roles.SelectMany(d => d.Accounts).Select(d => d.User);
        //        handlers = users1.Select(d => new Kv(d.Id, d.Name)).ToList();
        //        break;
        //    case EHandlerType.OrgLevel:
        //        //当前操作人所属部门的下级部门并且属于配置orgLevel的部门
        //        var levels = stepDefine.HandlerTypeItems.Select(d => d.Key).Select(d => int.Parse(d));
        //        var levelOneOrg = _sessionContext.RequiredOrgId.GetHigherOrgCode();
        //        //var orgs1 = await _organizeRepository.QueryAsync(d =>
        //        //    d.IsEnable && d.OrgCode.StartsWith(levelOneOrg) &&
        //        //    levels.Contains(d.OrgLevel));

        //        var orgs1 = await _organizeRepository.Queryable()
        //            .Where(d => d.IsEnable && levels.Contains(d.Level))
        //            .WhereIF(!levelOneOrg.IsCenter(), d => d.Id.StartsWith(levelOneOrg))
        //            .ToListAsync(cancellationToken);

        //        handlers = orgs1.Select(d => new Kv(d.Id, d.Name)).ToList();
        //        break;
        //    case EHandlerType.OrgType:
        //        var types = stepDefine.HandlerTypeItems.Select(d => d.Key)
        //            .Select(d => Enum.Parse<EOrgType>(d));
        //        var levelOneOrg1 = _sessionContext.RequiredOrgId.GetHigherOrgCode();
        //        //var org2 = await _organizeRepository.QueryAsync(d =>
        //        //    d.IsEnable && d.OrgCode.StartsWith(levelOneOrg1) &&
        //        //    types.Contains(d.OrgType));

        //        var orgs2 = await _organizeRepository.Queryable()
        //            .Where(d => d.IsEnable && types.Contains(d.OrgType))
        //            .WhereIF(!levelOneOrg1.IsCenter(), d => d.Id.StartsWith(levelOneOrg1))
        //            .ToListAsync(cancellationToken);

        //        handlers = orgs2.Select(d => new Kv(d.Id, d.Name)).ToList();
        //        break;
        //    default:
        //        throw new ArgumentOutOfRangeException();
        //}

        //var dto = new NextStepOptionDto { Handlers = handlers };

        //if (stepDefine.Components.Contains(SysDicTypeConsts.OrderRedoReason))
        //{
        //    var orderRedoReasons =
        //        await _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.OrderRedoReason,
        //            cancellationToken);
        //    dto.OrderRedoReasonOptions = orderRedoReasons.Select(d => new Kv(d.DicDataValue, d.DicDataName)).ToList();
        //}

        //return dto;
        throw new NotImplementedException();
    }

    /// <summary>
    /// 查询开始流程的下一步待选节点
    /// </summary>
    public async Task<NextStepsDto> GetStartStepsAsync(string moduleCode, CancellationToken cancellationToken)
    {
        var wfModule = await GetWorkflowModuleAsync(moduleCode, cancellationToken);
        var definition = wfModule.Definition;
        if (definition == null)
            throw new UserFriendlyException("无效模板编码");
        if (definition.Status is not EDefinitionStatus.Enable)
            throw new UserFriendlyException("该模板不可用");

        var startStep = definition.FindStartStepDefine();

        //var dto = new NextStepsDto
        //{
        //    DefinitionId = definition.Id,
        //    InstanceMode = startStep.InstanceMode,
        //    DynamicPolicy = startStep.InstancePolicy
        //};

        if (startStep.InstanceMode is EInstanceMode.Dynamic)
        {
            //var nextStepOption = CreateDynamicStep(startStep.InstancePolicy);
            //dto.Steps = new List<NextStepOption> { nextStepOption };
            //return dto;
            var nextStepOption = await CreateDynamicStepAsync(startStep.InstancePolicy.Value, cancellationToken);
            return new NextStepsDto
            {
                Steps = new List<NextStepOption> { nextStepOption }
            };
        }

        var firstStepDefines = definition.FindStepDefines(startStep.NextSteps.Select(d => d.Code));
        if (!firstStepDefines.Any())
            throw new UserFriendlyException("未正确配置首个办理节点");

        //var steps = firstStepDefines
        //        .Select(d => new NextStepOption { Key = d.Code, Value = d.Name })
        //        .ToList();
        return new NextStepsDto
        {
            Steps = await CreateConfigStepsAsync(firstStepDefines, cancellationToken)
        };
        //dto.Steps = steps;
        //return dto;
    }

    /// <summary>
    /// 查询办理流程的下一步待选节点
    /// </summary>
    public async Task<NextStepsDto> GetNextStepsAsync(string workflowId, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: cancellationToken);
        var currentStep = _workflowDomainService.FindCurrentStepWaitForHandle(workflow);
        if (currentStep.StepType is EStepType.End)
            throw new UserFriendlyException("结束节点无需办理");

        var dto = new NextStepsDto
        {
            CanReject = currentStep.CanReject,
            ExpiredTime = workflow.ExpiredTime,
        };

        if (currentStep.InstanceMode is EInstanceMode.Dynamic && !DynamicShouldTerminal(currentStep))
        {
            //动态生成下一步
            var nextStepOption = await CreateDynamicStepAsync(currentStep.InstancePolicy.Value, cancellationToken);
            dto.Steps = new List<NextStepOption> { nextStepOption };
            return dto;
        }

        if (currentStep.IsInCountersign())
        {
            var countersignEndOption = CreateCountersignEndStep(workflow.Steps, currentStep);

            if (currentStep.IsCountersignEndStep)
            {
                //当前待办节点为会签汇总节点时：检查是否为顶级会签发起节点(csstate==none)，t:按配置往下走，f:继续往上汇总，不需要重复往下指派
                if (currentStep.CountersignStartStepId != workflow.TopCountersignStepId)
                {
                    dto.Steps = new List<NextStepOption> { countersignEndOption };
                    return dto;
                }

                //var startCountersignStep = workflow.Steps.FirstOrDefault(d => d.Id == currentStep.CountersignStartStepId);
                //if (startCountersignStep is null)
                //    throw new UserFriendlyException(
                //        $"未查询到会签开始节点，workflowId: {workflow.Id}, startStepId: {currentStep.CountersignStartStepId}",
                //        "未查询到会签开始节点，数据异常");
                //if (startCountersignStep.CountersignPosition is not ECountersignPosition.None)
                //{
                //    var steps = new List<NextStepOption> { countersignEndOption };
                //    return new NextStepsDto { Steps = steps };
                //}
            }
            else
            {
                //按会签策略
                var nextStepOption = await CreateDynamicStepAsync(currentStep.CountersignPolicy.Value, cancellationToken);
                dto.Steps = new List<NextStepOption> { nextStepOption, countersignEndOption };
                return dto;
            }
        }

        var nextDefines = workflow.WorkflowDefinition.FindStepDefines(currentStep.NextSteps.Select(d => d.Code));
        if (!nextDefines.Any())
            throw new UserFriendlyException("未正确配置下一节点");

        dto.Steps = await CreateConfigStepsAsync(nextDefines, cancellationToken);
        return dto;


        //return new NextStepsDto
        //{
        //    Steps = nextDefines.Select(d => new NextStepOption { Key = d.Code, Value = d.Name }).ToList()
        //};
    }

    /// <summary>
    /// 查询撤回可选节点
    /// </summary>
    public async Task<NextStepsDto> GetRecallStepsAsync(string workflowId, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: cancellationToken);
        var stepCodes = workflow.Steps.Where(d => d.StepType != EStepType.End && d.IsOrigin)
            .Select(d => d.Code).ToList();
        var stepDefines = workflow.WorkflowDefinition.FindStepDefines(stepCodes);
        return new NextStepsDto
        {
            Steps = await CreateConfigStepsAsync(stepDefines, cancellationToken)
        };
    }

    /// <summary>
    /// 查询跳转可选节点
    /// </summary>
    public async Task<NextStepsDto> GetJumpStepsAsync(string workflowId, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, cancellationToken: cancellationToken);
        return new NextStepsDto
        {
            Steps = await CreateConfigStepsAsync(workflow.WorkflowDefinition.Steps, cancellationToken)
        };
    }

    /// <summary>
    /// 查询重办可选节点
    /// </summary>
    public async Task<NextStepsDto> GetRedoStepsAsync(string workflowId, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(workflowId, true, true, cancellationToken: cancellationToken);
        var steps = workflow.Steps.Where(d => d.StepType is EStepType.Normal && d.IsOrigin && d.IsOrg());
        var stepDefines = workflow.WorkflowDefinition.FindStepDefines(steps.Select(d => d.Code));
        return new NextStepsDto
        {
            Steps = await CreateConfigStepsAsync(stepDefines, cancellationToken)
        };
    }

    /// <summary>
    /// 否决
    /// </summary>
    public async Task RejectAsync(RejectDto dto, CancellationToken cancellationToken)
    {
        var workflow = await _workflowDomainService.GetWorkflowAsync(dto.WorkflowId, true, true,
                cancellationToken: cancellationToken);
        var basicDto = _mapper.Map<BasicWorkflowDto>(dto);
        basicDto.NextStepCode = string.Empty;
        basicDto.IsStartCountersign = false;
        await _workflowDomainService.RejectAsync(workflow, basicDto, cancellationToken);
    }

    private async Task<List<NextStepOption>> CreateConfigStepsAsync(List<StepDefine> stepDefines, CancellationToken cancellationToken)
    {
        var stepOptions = new List<NextStepOption>();
        foreach (var stepDefine in stepDefines)
        {
            var handlers = new List<Kv>();
            if (stepDefine.StepType is EStepType.End)
            {
                stepOptions.Add(new NextStepOption
                {
                    Key = stepDefine.Code,
                    Value = stepDefine.Name,
                    Items = handlers
                });
                continue;
            }

            var levelOneOrgCode = _sessionContext.RequiredOrgId.GetHigherOrgCode();
            switch (stepDefine.HandlerType)
            {
                case EHandlerType.AssignedUser:
                case EHandlerType.AssignedOrg:
                    handlers = stepDefine.HandlerTypeItems;
                    break;
                case EHandlerType.Role:
                    //当前操作人所属部门的下级部门并且属于配置包含角色
                    var roles = await _roleRepository.Queryable()
                        .Includes(
                            d => d.Accounts.Where(
                                x => !x.IsDeleted && x.Status == EAccountStatus.Normal).ToList(),
                            x => x.User)
                        .Where(d => stepDefine.HandlerTypeItems.Select(x => x.Key).Contains(d.Name))
                        .ToListAsync(cancellationToken);
                    var users1 = roles.SelectMany(d => d.Accounts).Select(d => d.User);

                    //解决当前为一级部门选择中心汇总
                    if (stepDefine.StepType != EStepType.Summary || stepDefine.BusinessType != EBusinessType.Center)
                        users1 = users1.Where(d => d.OrgId.StartsWith(levelOneOrgCode));

                    handlers = users1.Select(d => new Kv(d.Id, d.Name)).ToList();
                    break;
                case EHandlerType.OrgLevel:
                    //当前操作人所属部门的垂直部门并且属于配置orgLevel的部门
                    var levels = stepDefine.HandlerTypeItems.Select(d => d.Key).Select(d => int.Parse(d));
                    var orgs1 = await _organizeRepository.Queryable()
                        .Where(d => d.IsEnable && levels.Contains(d.Level))
                        .WhereIF(!levelOneOrgCode.IsCenter(), d => d.Id.StartsWith(levelOneOrgCode))
                        .ToListAsync(cancellationToken);

                    handlers = orgs1.Select(d => new Kv(d.Id, d.Name)).ToList();
                    break;
                case EHandlerType.OrgType:
                    var types = stepDefine.HandlerTypeItems.Select(d => d.Key)
                        .Select(d => Enum.Parse<EOrgType>(d));
                    var orgs2 = await _organizeRepository.Queryable()
                        .Where(d => d.IsEnable && types.Contains(d.OrgType))
                        .WhereIF(!levelOneOrgCode.IsCenter(), d => d.Id.StartsWith(levelOneOrgCode))
                        .ToListAsync(cancellationToken);

                    handlers = orgs2.Select(d => new Kv(d.Id, d.Name)).ToList();
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            stepOptions.Add(new NextStepOption
            {
                Key = stepDefine.Code,
                Value = stepDefine.Name,
                Items = handlers
            });
        }

        return stepOptions;
    }

    private bool DynamicShouldTerminal(WorkflowStep step)
    {
        if (step.InstanceMode is not EInstanceMode.Dynamic)
            throw new UserFriendlyException("非动态节点");
        switch (step.InstancePolicy)
        {
            case EDynamicPolicy.OrgUpCenterTop:
            case EDynamicPolicy.OrgUp:
            case EDynamicPolicy.OrgDownCenterTop:
            case EDynamicPolicy.OrgDown:
                return step.DynamicShouldTerminal();
            default:
                throw new ArgumentOutOfRangeException();
        }
    }

    /// <summary>
    /// 查询下一步所选节点对应待选项
    /// </summary>
    public async Task<IReadOnlyList<Kv>> GetNextStepOptionsAsync(GetNextStepItemsDto dto, CancellationToken cancellationToken)
    {
        List<Kv> nextStepOptions;
        if (dto.InstanceMode is EInstanceMode.Config)
        {
            nextStepOptions = await NextStepOptionsByConfigAsync(dto, cancellationToken);
        }
        else
        {
            nextStepOptions = await CreateNextStepOptionsDynamicAsync(dto, cancellationToken);
        }

        return nextStepOptions;
    }

    private async Task<List<Kv>> CreateNextStepOptionsDynamicAsync(GetNextStepItemsDto dto, CancellationToken cancellationToken)
    {
        if (!int.TryParse(dto.Step.Key, out int orgLevel))
            throw new UserFriendlyException("无效参数");
        List<Kv> items;
        var firstLevelOrgCode = _sessionContext.RequiredOrgId.GetHigherOrgCode();

        if (dto.Step.BackToCountersignEnd)
        {
            //todo 限定返回发起会签的办理人
            //items = 
        }

        switch (dto.DynamicPolicy)
        {
            case EDynamicPolicy.OrgUpCenterTop:
                if (orgLevel == 0)
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => d.IsCenter)
                        .Select(d => new Kv(d.Id, d.Name))
                        .ToListAsync(cancellationToken);
                }
                else
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel && d.Id.StartsWith(firstLevelOrgCode))
                        .Select(d => new Kv(d.Id, d.Name))
                        .ToListAsync(cancellationToken);
                }
                break;
            case EDynamicPolicy.OrgUp:
                items = await _organizeRepository.Queryable()
                    .Where(d => d.Level == orgLevel && d.Id.StartsWith(firstLevelOrgCode))
                    .Select(d => new Kv(d.Id, d.Name))
                    .ToListAsync(cancellationToken);
                break;
            case EDynamicPolicy.OrgDownCenterTop:
                if (_sessionContext.OrgIsCenter)
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel)
                        .Select(d => new Kv(d.Id, d.Name))
                        .ToListAsync(cancellationToken);
                }
                else
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel && d.Id.StartsWith(firstLevelOrgCode))
                        .Select(d => new Kv(d.Id, d.Name))
                        .ToListAsync(cancellationToken);
                }
                break;
            case EDynamicPolicy.OrgDown:
                items = await _organizeRepository.Queryable()
                    .Where(d => d.Level == orgLevel && d.Id.StartsWith(firstLevelOrgCode))
                    .Select(d => new Kv(d.Id, d.Name))
                    .ToListAsync(cancellationToken);
                break;

            default:
                throw new ArgumentOutOfRangeException();
        }

        return items;
    }

    private async Task<List<Kv>> NextStepOptionsByConfigAsync(GetNextStepItemsDto dto, CancellationToken cancellationToken)
    {
        var definition = await _definitionRepository.GetAsync(dto.DefinitionId, cancellationToken);
        if (definition == null)
            throw new UserFriendlyException("无效DefineId");

        var stepDefine = definition.FindStepDefine(dto.Step.Key);
        if (stepDefine is null)
            throw UserFriendlyException.SameMessage("未查询到对应节点配置");

        if (stepDefine.StepType == EStepType.End)
            throw new UserFriendlyException("结束节点无待选项");

        var handlers = new List<Kv>();
        var firstLevelOrgCode = _sessionContext.RequiredOrgId.GetHigherOrgCode();
        switch (stepDefine.HandlerType)
        {
            case EHandlerType.AssignedUser:
            case EHandlerType.AssignedOrg:
                handlers = stepDefine.HandlerTypeItems;
                break;
            case EHandlerType.Role:
                //当前操作人所属部门的下级部门并且属于配置包含角色
                var roles = await _roleRepository.Queryable()
                    .Includes(
                        d => d.Accounts.Where(x =>
                            !x.IsDeleted && x.Status == EAccountStatus.Normal &&
                            x.User.OrgId.StartsWith(firstLevelOrgCode)).ToList(),
                        x => x.User)
                    .Where(d => stepDefine.HandlerTypeItems.Select(x => x.Key).Contains(d.Name))
                    .ToListAsync(cancellationToken);
                var users1 = roles.SelectMany(d => d.Accounts).Select(d => d.User);
                handlers = users1.Select(d => new Kv(d.Id, d.Name)).ToList();
                break;
            case EHandlerType.OrgLevel:
                //当前操作人所属部门的垂直部门并且属于配置orgLevel的部门
                var levels = stepDefine.HandlerTypeItems.Select(d => d.Key).Select(d => int.Parse(d));
                var orgs1 = await _organizeRepository.Queryable()
                    .Where(d => d.IsEnable && levels.Contains(d.Level))
                    .WhereIF(!firstLevelOrgCode.IsCenter(), d => d.Id.StartsWith(firstLevelOrgCode))
                    .ToListAsync(cancellationToken);

                handlers = orgs1.Select(d => new Kv(d.Id, d.Name)).ToList();
                break;
            case EHandlerType.OrgType:
                var types = stepDefine.HandlerTypeItems.Select(d => d.Key)
                    .Select(d => Enum.Parse<EOrgType>(d));
                var orgs2 = await _organizeRepository.Queryable()
                    .Where(d => d.IsEnable && types.Contains(d.OrgType))
                    .WhereIF(!firstLevelOrgCode.IsCenter(), d => d.Id.StartsWith(firstLevelOrgCode))
                    .ToListAsync(cancellationToken);

                handlers = orgs2.Select(d => new Kv(d.Id, d.Name)).ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException();
        }

        return handlers;
    }


    private NextStepOption CreateCountersignEndStep(List<WorkflowStep> steps, WorkflowStep step)
    {
        var parentStep = steps.FirstOrDefault(d => d.Id == step.PrevStepId);
        if (parentStep is null)
            throw new UserFriendlyException("未查找到会签上级节点");
        var text = parentStep.HandlerOrgIsCenter.Value
            ? "热线中心汇总"
            : $"{parentStep.HandlerOrgId.CalcOrgLevel().ToChinese()}级部门汇总";
        return new NextStepOption
        {
            Key = parentStep.Code,
            Value = text,//parentStep.Name,//todo name不对，目前为definition.name,需改为x级部门办理
            BackToCountersignEnd = true,
            Items = new List<Kv> { new(parentStep.HandlerId, parentStep.HandlerName) },
        };
    }

    /// <summary>
    /// 依据当前操作人及动态策略生成下一步节点
    /// </summary>
    private NextStepOption CreateDynamicStep(EDynamicPolicy policy)
    {
        int orgLevel;
        switch (policy)
        {
            case EDynamicPolicy.OrgUpCenterTop:
                orgLevel = _sessionContext.OrgLevel - 1;
                if (orgLevel < 0) orgLevel = 0;
                break;
            case EDynamicPolicy.OrgUp:
                orgLevel = _sessionContext.OrgLevel - 1;
                if (orgLevel <= 0) orgLevel = 1;
                break;
            case EDynamicPolicy.OrgDownCenterTop:
                orgLevel = _sessionContext.OrgIsCenter
                    ? 1
                    : _sessionContext.OrgLevel + 1;
                break;
            case EDynamicPolicy.OrgDown:
                orgLevel = _sessionContext.OrgLevel + 1;
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(policy), policy, null);
        }

        return new NextStepOption
        {
            Key = orgLevel.ToString(),
            Value = orgLevel == 0 ? "热线中心" : $"{orgLevel.ToChinese()}级部门",
        };
    }

    private async Task<NextStepOption> CreateDynamicStepAsync(EDynamicPolicy policy, CancellationToken cancellationToken)
    {
        int orgLevel;
        List<Kv> items;
        var levelOneOrgCode = _sessionContext.RequiredOrgId.GetHigherOrgCode();
        switch (policy)
        {
            case EDynamicPolicy.OrgUpCenterTop:
                orgLevel = _sessionContext.OrgLevel - 1;
                if (orgLevel < 0) orgLevel = 0;

                if (orgLevel == 0)
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => d.IsCenter)
                        .Select(d => new Kv { Key = d.Id, Value = d.Name })
                        .ToListAsync(cancellationToken);
                }
                else
                {
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel && d.Id.StartsWith(levelOneOrgCode))
                        .Select(d => new Kv { Key = d.Id, Value = d.Name })
                        .ToListAsync(cancellationToken);
                }
                break;
            case EDynamicPolicy.OrgUp:
                orgLevel = _sessionContext.OrgLevel - 1;
                if (orgLevel <= 0) orgLevel = 1;
                items = await _organizeRepository.Queryable()
                    .Where(d => d.Level == orgLevel && d.Id.StartsWith(levelOneOrgCode))
                    .Select(d => new Kv { Key = d.Id, Value = d.Name })
                    .ToListAsync(cancellationToken);
                break;
            case EDynamicPolicy.OrgDownCenterTop:
                if (_sessionContext.OrgIsCenter)
                {
                    orgLevel = 1;
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel)
                        .Select(d => new Kv { Key = d.Id, Value = d.Name })
                        .ToListAsync(cancellationToken);
                }
                else
                {
                    orgLevel = _sessionContext.OrgLevel + 1;
                    items = await _organizeRepository.Queryable()
                        .Where(d => !d.IsCenter && d.Level == orgLevel && d.Id.StartsWith(levelOneOrgCode))
                        .Select(d => new Kv { Key = d.Id, Value = d.Name })
                        .ToListAsync(cancellationToken);
                }
                break;
            case EDynamicPolicy.OrgDown:
                orgLevel = _sessionContext.OrgLevel + 1;
                items = await _organizeRepository.Queryable()
                    .Where(d => d.Level == orgLevel && d.Id.StartsWith(levelOneOrgCode))
                    .Select(d => new Kv { Key = d.Id, Value = d.Name })
                    .ToListAsync(cancellationToken);
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(policy), policy, null);
        }

        return new NextStepOption
        {
            Key = orgLevel.ToString(),
            Value = orgLevel == 0 ? "热线中心" : $"{orgLevel.ToChinese()}级部门",
            Items = items
        };
    }

    //private async Task<IReadOnlyList<NextStepOption>> CreateConfigStepsAsync(List<StepDefine> stepDefines,
    //    CancellationToken cancellationToken)
    //{
    //    foreach (var stepDefine in stepDefines)
    //    {
    //        if (stepDefine.StepType == EStepType.End)
    //            throw new UserFriendlyException("结束节点无待选项");

    //        var handlers = new List<Kv>();
    //        switch (stepDefine.HandlerType)
    //        {
    //            case EHandlerType.AssignedUser:
    //            case EHandlerType.AssignedOrg:
    //                handlers = stepDefine.HandlerTypeItems;
    //                break;
    //            case EHandlerType.Role:
    //                //当前操作人所属部门的下级部门并且属于配置包含角色
    //                var roles = await _roleRepository.Queryable()
    //                    .Includes(d => d.Accounts.Where(x => !x.IsDeleted && x.Status == EAccountStatus.Normal).ToList(),
    //                        x => x.User)
    //                    .Where(d => stepDefine.HandlerTypeItems.Select(x => x.Key).Contains(d.Name))
    //                    .ToListAsync(cancellationToken);
    //                var users1 = roles.SelectMany(d => d.Accounts).Select(d => d.User);
    //                handlers = users1.Select(d => new Kv(d.Id, d.Name)).ToList();
    //                break;
    //            case EHandlerType.OrgLevel:
    //                //当前操作人所属部门的下级部门并且属于配置orgLevel的部门
    //                var levels = stepDefine.HandlerTypeItems.Select(d => d.Key).Select(d => int.Parse(d));
    //                var levelOneOrg = _sessionContext.RequiredOrgId.GetHigherOrgCode();
    //                //var orgs1 = await _organizeRepository.QueryAsync(d =>
    //                //    d.IsEnable && d.OrgCode.StartsWith(levelOneOrg) &&
    //                //    levels.Contains(d.OrgLevel));

    //                var orgs1 = await _organizeRepository.Queryable()
    //                    .Where(d => d.IsEnable && levels.Contains(d.Level))
    //                    .WhereIF(!levelOneOrg.IsCenter(), d => d.Id.StartsWith(levelOneOrg))
    //                    .ToListAsync(cancellationToken);

    //                handlers = orgs1.Select(d => new Kv(d.Id, d.Name)).ToList();
    //                break;
    //            case EHandlerType.OrgType:
    //                var types = stepDefine.HandlerTypeItems.Select(d => d.Key)
    //                    .Select(d => Enum.Parse<EOrgType>(d));
    //                var levelOneOrg1 = _sessionContext.RequiredOrgId.GetHigherOrgCode();
    //                //var org2 = await _organizeRepository.QueryAsync(d =>
    //                //    d.IsEnable && d.OrgCode.StartsWith(levelOneOrg1) &&
    //                //    types.Contains(d.OrgType));

    //                var orgs2 = await _organizeRepository.Queryable()
    //                    .Where(d => d.IsEnable && types.Contains(d.OrgType))
    //                    .WhereIF(!levelOneOrg1.IsCenter(), d => d.Id.StartsWith(levelOneOrg1))
    //                    .ToListAsync(cancellationToken);

    //                handlers = orgs2.Select(d => new Kv(d.Id, d.Name)).ToList();
    //                break;
    //            default:
    //                throw new ArgumentOutOfRangeException();
    //        }

    //        var dto = new NextStepOptionDto { Handlers = handlers };

    //        if (stepDefine.Components.Contains(SysDicTypeConsts.OrderRedoReason))
    //        {
    //            var orderRedoReasons =
    //                await _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.OrderRedoReason,
    //                    cancellationToken);
    //            dto.OrderRedoReasonOptions = orderRedoReasons.Select(d => new Kv(d.DicDataValue, d.DicDataName)).ToList();
    //        }

    //        return dto;
    //    }
    //    throw new NotImplementedException();
    //}

    /// <summary>
    /// 查询指派办理人的处理方式及实际办理人
    /// </summary>
    public async Task<FlowAssignInfo> GetFlowAssignInfoAsync(StepDefine stepDefine, bool isStartCountersign,
        List<Kv> handlers, CancellationToken cancellationToken)
    {
        if (stepDefine.StepType is EStepType.End) return new();

        if (isStartCountersign)
        {
            return FlowAssignInfo.Create(EFlowAssignType.Org, handlers, isStartCountersign);
        }
        else
        {
            switch (stepDefine.InstanceMode)
            {
                case EInstanceMode.Config:
                    switch (stepDefine.HandlerType)
                    {
                        case EHandlerType.Role:
                            if (!handlers.Any())
                            {
                                var roles = await _roleRepository.Queryable()
                                    .Includes(d => d.Accounts, x => x.User)
                                    .Where(d => stepDefine.HandlerTypeItems.Select(d => d.Key).Contains(d.Name))
                                    .ToListAsync(cancellationToken);
                                handlers = roles.SelectMany(d => d.Accounts).Distinct()
                                    .Select(d => new Kv(d.Id, d.User.Name))
                                    .ToList();
                            }

                            return FlowAssignInfo.Create(EFlowAssignType.User, handlers, isStartCountersign);

                        case EHandlerType.OrgLevel:
                        case EHandlerType.OrgType:
                        case EHandlerType.AssignedOrg:
                            return FlowAssignInfo.Create(EFlowAssignType.Org, handlers, isStartCountersign);

                        case EHandlerType.AssignedUser:
                            return FlowAssignInfo.Create(EFlowAssignType.User, handlers, isStartCountersign);

                        default:
                            throw new ArgumentOutOfRangeException();
                    }
                case EInstanceMode.Dynamic:
                    return FlowAssignInfo.Create(EFlowAssignType.Org, handlers, isStartCountersign);
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

    }


    #region private

    /// <summary>
    /// 查询流程业务模块
    /// </summary>
    /// <param name="code"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    private async Task<WorkflowModule> GetWorkflowModuleAsync(string code, CancellationToken cancellationToken)
    {
        var wfModule = await _wfModuleCacheManager.GetWorkflowModuleAsync(code, cancellationToken);
        if (wfModule == null)
            throw UserFriendlyException.SameMessage("无效流程模块编码");
        if (wfModule.Definition is null)
            throw new UserFriendlyException($"{code} 未配置流程模板", "未配置流程模板");
        return wfModule;
    }

    /// <summary>
    /// 更新接办部门
    /// </summary>
    private async Task AddOrUpdateAssignAsync(Workflow workflow, BasicWorkflowDto dto, StepDefine nextStepBoxDefine,
        CancellationToken cancellationToken)
    {
        if (nextStepBoxDefine.StepType is EStepType.Normal)
        {
            await _workflowAssignRepository.RemoveAsync(d =>
                    d.WorkflowId == workflow.Id && d.OrgCode == _sessionContext.RequiredOrgId,
                cancellationToken: cancellationToken);

            var assigns = new List<WorkflowAssign>();
            switch (nextStepBoxDefine.HandlerType)
            {
                case EHandlerType.Role:
                    if (dto.NextHandlers.Any())
                    {
                        //选了handler，handler为userId 
                        var users1 = await _userRepository.Queryable()
                            .Includes(d => d.Organization)
                            .Where(d => dto.NextHandlers.Select(x => x.Key).Contains(d.Id))
                            .ToListAsync(cancellationToken);
                        assigns = users1.Select(d => WorkflowAssign.Create(workflow.Id, d.Id, d.Organization.Name))
                            .ToList();
                    }
                    else
                    {
                        ////如果模板配置为本部门办理，则为当前办理人orgCode，非本部门办理:属于该角色的所有用户所属部门
                        //if (nextStepBoxDefine.OnlySelfOrg.HasValue && nextStepBoxDefine.OnlySelfOrg.Value)
                        //{
                        //    assigns = new List<WorkflowAssign>
                        //    {
                        //        WorkflowAssign.Create(workflow.Id, _sessionContext.RequiredOrgCode, _sessionContext.OrgName)
                        //    };
                        //}
                        //else
                        //{
                        var accounts = await _accountRepository.Queryable()
                            .Includes(d => d.User, d => d.Organization)
                            .Where(d => dto.NextHandlers.Select(d => d.Key).Contains(d.Name))
                            .ToListAsync(cancellationToken);
                        assigns = accounts.Select(d => d.User.Organization).Select(d =>
                            WorkflowAssign.Create(workflow.Id, d.Id, d.Name)).ToList();
                        //}
                    }

                    break;

                case EHandlerType.OrgLevel:
                case EHandlerType.OrgType:
                case EHandlerType.AssignedOrg:
                    assigns = dto.NextHandlers.Select(d => WorkflowAssign.Create(workflow.Id, d.Key, d.Value)).ToList();
                    break;

                case EHandlerType.AssignedUser:
                    //指定人所属部门
                    var users = await _userRepository.Queryable()
                        .Includes(d => d.Organization)
                        .Where(d => dto.NextHandlers.Select(x => x.Key).Contains(d.Id))
                        .ToListAsync(cancellationToken);
                    assigns = users.Select(d => WorkflowAssign.Create(workflow.Id, d.Id, d.Organization.Name))
                        .ToList();
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            if (assigns.Any())
                await _workflowAssignRepository.AddRangeAsync(assigns, cancellationToken);
        }
    }

    #endregion
}