﻿using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Settings.Hotspots;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using Hotline.Users;
using MapsterMapper;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace Hotline.Application.Knowledge
{
	public class KnowApplication : IKnowApplication, IScopeDependency
	{
		private readonly IKnowledgeRepository _knowledgeRepository;
		private readonly IRepository<KnowledgeApply> _knowledgeApplyRepository;
		private readonly ISessionContext _sessionContext;
		private readonly IMapper _mapper;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="knowledgeRepository"></param>
		/// <param name="knowledgeApplyRepository"></param>
		/// <param name="sessionContext"></param>
		/// <param name="knowledgeDomainService"></param>
		/// <param name="mapper"></param>
		public KnowApplication(IKnowledgeRepository knowledgeRepository, IRepository<KnowledgeApply> knowledgeApplyRepository, ISessionContext sessionContext, IMapper mapper)
		{
			_knowledgeRepository = knowledgeRepository;
			_knowledgeApplyRepository = knowledgeApplyRepository;
			_sessionContext = sessionContext;
			_mapper = mapper;
		}

		/// <summary>
		/// 知识库查询
		/// </summary>
		/// <param name="pagedDto"></param>
		/// <returns></returns>
		public async Task<PagedDto<KnowledgeDataDto>> GetKnowList(KnowPagedListDto pagedDto)
		{
			RefAsync<int> total = 0;
			var temp = await _knowledgeRepository
			   .Queryable()
			   .Includes(x => x.User)
			   .Includes(x => x.SystemOrganize)
			   .Includes(x => x.KnowledgeType)
			   .Includes(x => x.HotspotType)
			   .Where(o => o.IsDeleted == false)
			   .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), x => x.Title.Contains(pagedDto.Title!))
			   .WhereIF(pagedDto.Status.HasValue, x => x.Status == pagedDto.Status)
			   .WhereIF(pagedDto.IsPublic.HasValue, x => x.IsPublic == pagedDto.IsPublic)
			   .WhereIF(!string.IsNullOrEmpty(pagedDto.Summary), x => x.Summary != null && x.Summary.Contains(pagedDto.Summary!))
			   .OrderByDescending(d => d.CreationTime)
			  //转分页数据
			  .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);
			//返回数据
			return new PagedDto<KnowledgeDataDto>(total.Value, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(temp));
		}

		/// <summary>
		/// 知识申请查询
		/// </summary>
		/// <param name="pagedDto"></param>
		/// <param name="cancellationToken"></param>
		/// <returns></returns>
		public async Task<PagedDto<KnowledgeApplyHandlePageDto>> GetApplyList(KnowledgeApplyPagedDto pagedDto, CancellationToken cancellationToken)
		{
			var (total, items) = await _knowledgeApplyRepository
			   .Queryable()
			   .Includes(it => it.User)
			   .Includes(it => it.SystemOrganize)
			   .Where(d => d.CreatorId == _sessionContext.RequiredUserId)
			   .WhereIF(pagedDto.Status.HasValue, d => d.Status == pagedDto.Status)
			   .WhereIF(pagedDto.ApplyType.HasValue, d => d.KnowledgeApplyType == pagedDto.ApplyType)
			   .WhereIF(!string.IsNullOrEmpty(pagedDto.IsOvertime) && pagedDto.IsOvertime == "0", d => d.IsOvertime == true)
			   .WhereIF(!string.IsNullOrEmpty(pagedDto.IsOvertime) && pagedDto.IsOvertime == "1", d => d.IsOvertime == false)
			   .WhereIF(pagedDto.StartTime.HasValue, d => d.CreationTime >= pagedDto.StartTime)
			   .WhereIF(pagedDto.EndTime.HasValue, d => d.CreationTime <= pagedDto.EndTime)
			   .OrderByDescending(p => p.CreationTime)
			   .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, cancellationToken);

			return new PagedDto<KnowledgeApplyHandlePageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyHandlePageDto>>(items));
		}

		/// <summary>
		/// 申请处理查询
		/// </summary>
		/// <param name="pagedDto"></param>
		/// <param name="cancellationToken"></param>
		/// <returns></returns>
		public async Task<PagedDto<KnowledgeApplyHandlePageDto>> GetApplyHandleList(KnowledgeHandlePagedDto pagedDto, CancellationToken cancellationToken)
		{
			var (total, items) = await _knowledgeApplyRepository
				 .Queryable()
				 .Includes(it => it.User)
				 .Includes(it => it.SystemOrganize)
				 .Where(p => p.DepartmentId == _sessionContext.RequiredOrgId)
				 .WhereIF(pagedDto.Status.HasValue, d => d.Status == pagedDto.Status)
				 .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.User.Name.Contains(pagedDto.Keyword!) || d.SystemOrganize.Name.Contains(pagedDto.Keyword!))
				 .WhereIF(pagedDto.ApplyType.HasValue, d => d.KnowledgeApplyType == pagedDto.ApplyType)
				 .WhereIF(!string.IsNullOrEmpty(pagedDto.IsOvertime) && pagedDto.IsOvertime == "0", d => d.IsOvertime == true)
				 .WhereIF(!string.IsNullOrEmpty(pagedDto.IsOvertime) && pagedDto.IsOvertime == "1", d => d.IsOvertime == false)

				 .WhereIF(pagedDto.StartTime.HasValue && pagedDto.Status != EKnowledgeApplyStatus.Failed && pagedDto.Status != EKnowledgeApplyStatus.Succeed, d => d.CreationTime >= pagedDto.StartTime)
				 .WhereIF(pagedDto.EndTime.HasValue && pagedDto.Status != EKnowledgeApplyStatus.Failed && pagedDto.Status != EKnowledgeApplyStatus.Succeed, d => d.CreationTime <= pagedDto.EndTime)
				 .WhereIF(pagedDto.StartTime.HasValue && pagedDto.Status == EKnowledgeApplyStatus.Failed, d => d.ReturnTime >= pagedDto.StartTime)
				 .WhereIF(pagedDto.EndTime.HasValue && pagedDto.Status == EKnowledgeApplyStatus.Failed, d => d.ReturnTime <= pagedDto.EndTime)
				  .WhereIF(pagedDto.StartTime.HasValue && pagedDto.Status == EKnowledgeApplyStatus.Succeed, d => d.HandleTime >= pagedDto.StartTime)
				 .WhereIF(pagedDto.EndTime.HasValue && pagedDto.Status == EKnowledgeApplyStatus.Succeed, d => d.HandleTime <= pagedDto.EndTime)

				 .OrderByDescending(p => p.CreationTime)
				 .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, cancellationToken);

			return new PagedDto<KnowledgeApplyHandlePageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyHandlePageDto>>(items));
		}
	}
}
